/***
 * Cecilia ADL Compiler
 * Copyright (C) 2006-2007 STMicroelectronics
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@objectweb.org
 *
 * Author:Matthieu Leclercq
 */

package org.objectweb.fractal.cecilia.adl.compiler;

import java.io.File;

import org.objectweb.fractal.cecilia.adl.FileProvider;
import org.objectweb.fractal.cecilia.adl.SourceFileProvider;
import org.objectweb.fractal.task.codegen.util.SourceFile;
import org.objectweb.fractal.task.core.Executable;
import org.objectweb.fractal.task.core.primitive.annotations.ClientInterface;
import org.objectweb.fractal.task.core.primitive.annotations.ServerInterface;
import org.objectweb.fractal.task.core.primitive.annotations.ServerInterfaces;
import org.objectweb.fractal.task.core.primitive.annotations.TaskParameters;

@TaskParameters("id")
@ServerInterfaces(@ServerInterface(name = "file-provider", signature = FileProvider.class, record = "role:compiledFile, id:%", parameters = "id"))
public abstract class AbstractCompilationTask
    implements
      Executable,
      FileProvider {

  /** The default file extension of compiled files. */
  protected static final String DEFAULT_EXTENSION = ".o";

  /** The compilation flags. */
  protected final String        flags;
  /** The output directory. */
  protected final File          outputDir;

  // Produced file
  protected File                compiledFile;

  // ---------------------------------------------------------------------------
  // Task client interfaces
  // ---------------------------------------------------------------------------

  /** The client interface used to retrieve the source file to compile. */
  @ClientInterface(name = "source-file-provider", record = "role:sourceFile, id:%", parameters = "id")
  public SourceFileProvider     sourceFileProviderItf;

  // ---------------------------------------------------------------------------
  // Task constructor
  // ---------------------------------------------------------------------------

  /**
   * @param outputDir the directory into which the compiled file will be placed.
   * @param flags compilation flags.
   */
  public AbstractCompilationTask(final File outputDir, final String flags) {
    this.outputDir = outputDir;
    this.flags = flags;
  }

  // ---------------------------------------------------------------------------
  // Abstract methods
  // ---------------------------------------------------------------------------

  /**
   * Compilation method.
   * 
   * @param sourceFile the source file to compile.
   * @param outputFile the compiled file that is provided by this task.
   * @throws Exception if an error occurs.
   */
  protected abstract void compile(SourceFile sourceFile, File outputFile)
      throws Exception;

  // ---------------------------------------------------------------------------
  // Utility methods
  // ---------------------------------------------------------------------------

  protected File getOutputFile(final SourceFile sourceFile) {
    final String fileName = sourceFile.getSignature().replace('.',
        File.separatorChar)
        + getExtension();
    return new File(outputDir, fileName);
  }

  protected String getExtension() {
    return DEFAULT_EXTENSION;
  }

  protected void createParentDir(final File outputFile)
      throws CompilationException {
    final File parent = outputFile.getParentFile();
    if (parent.exists()) {
      if (!parent.isDirectory()) {
        throw new CompilationException("Invalid output file '" + outputFile
            + "': the parent file is not a directory");
      }
    } else {
      if (!parent.mkdirs()) {
        throw new CompilationException(
            "Can't create parent directory of file '" + outputFile + "'");
      }
    }
  }

  // -------------------------------------------------------------------------
  // Implementation of the Executable interface
  // -------------------------------------------------------------------------

  public void execute() throws Exception {
    final SourceFile sourceFile = sourceFileProviderItf.getSourceFile();
    compiledFile = getOutputFile(sourceFile);
    createParentDir(compiledFile);
    compile(sourceFile, compiledFile);
  }

  // -------------------------------------------------------------------------
  // Implementation of the FileProvider interface
  // -------------------------------------------------------------------------

  public File getFile() {
    if (compiledFile == null) try {
      execute();
    } catch (final Exception e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
    return compiledFile;
  }
}
