/***
 * Cecilia ADL Compiler
 * Copyright (C) 2006-2007 STMicroelectronics
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@objectweb.org
 *
 * Author:Matthieu Leclercq
 */

package org.objectweb.fractal.cecilia.adl.compiler;

import java.io.File;
import java.util.Collection;
import java.util.HashMap;
import java.util.LinkedHashSet;
import java.util.Map;

import org.objectweb.fractal.cecilia.adl.FileCollectionProvider;
import org.objectweb.fractal.cecilia.adl.FileProvider;
import org.objectweb.fractal.task.core.Executable;
import org.objectweb.fractal.task.core.primitive.annotations.ClientInterface;
import org.objectweb.fractal.task.core.primitive.annotations.ServerInterface;
import org.objectweb.fractal.task.core.primitive.annotations.ServerInterfaces;
import org.objectweb.fractal.task.core.primitive.annotations.TaskParameters;

@TaskParameters("id")
@ServerInterfaces(@ServerInterface(name = "file-provider", signature = FileProvider.class, record = "role:linkedFile, id:%", parameters = "id"))
public abstract class AbstractLinkTask implements Executable, FileProvider {

  /** The name of the linked file. */
  protected final String                           outputName;
  /** The compilation flags. */
  protected final String                           flags;
  /** The output directory. */
  protected final File                             outputDir;

  // Produced file
  protected File                                   linkedFile;

  // ---------------------------------------------------------------------------
  // Task client interfaces
  // ---------------------------------------------------------------------------

  /** The client collection interface used to retrieve the files to linked. */
  @ClientInterface(name = "source-file-provider", signature = FileProvider.class, record = "role:compiledFile, id:%", parameters = "id")
  public final Map<String, FileProvider>           fileProviderItfs           = new HashMap<String, FileProvider>();

  /** The client collection interface used to retrieve the files to linked. */
  @ClientInterface(name = "source-file-collection-provider", signature = FileCollectionProvider.class, record = "role:compiledFileCollection, id:%", parameters = "id")
  public final Map<String, FileCollectionProvider> fileCollectionProviderItfs = new HashMap<String, FileCollectionProvider>();

  // ---------------------------------------------------------------------------
  // Task constructor
  // ---------------------------------------------------------------------------

  /**
   * @param outputName the name of the linked file
   * @param outputDir the directory into which the compiled file will be placed.
   * @param flags compilation flags.
   */
  public AbstractLinkTask(final String outputName, final File outputDir,
      final String flags) {
    this.outputName = outputName;
    this.outputDir = outputDir;
    this.flags = flags;
  }

  // ---------------------------------------------------------------------------
  // Abstract methods
  // ---------------------------------------------------------------------------

  /**
   * Link method.
   * 
   * @param outputFile the linked file that is provided by this task.
   * @param inputFiles the input files to be linked.
   * @throws Exception if an error occurs.
   */
  protected abstract void link(File outputFile, Collection<File> inputFiles)
      throws Exception;

  // ---------------------------------------------------------------------------
  // Utility methods
  // ---------------------------------------------------------------------------

  protected File getOutputFile() {
    return new File(outputDir, outputName);
  }

  protected void createParentDir(final File outputFile)
      throws CompilationException {
    final File parent = outputFile.getParentFile();
    if (parent.exists()) {
      if (!parent.isDirectory()) {
        throw new CompilationException("Invalid output file '" + outputFile
            + "': the parent file is not a directory");
      }
    } else {
      if (!parent.mkdirs()) {
        throw new CompilationException(
            "Can't create parent directory of file '" + outputFile + "'");
      }
    }
  }

  // -------------------------------------------------------------------------
  // Implementation of the Executable interface
  // -------------------------------------------------------------------------

  public void execute() throws Exception {
    // use LinkedHashSet to preserve file order.
    final Collection<File> inputFiles = new LinkedHashSet<File>();

    // get every files provided by every FileProvider
    for (final FileProvider fileProvider : fileProviderItfs.values()) {
      inputFiles.add(fileProvider.getFile());
    }

    // get every files provided by every FileCollectionProvider
    for (final FileCollectionProvider collectionProvider : fileCollectionProviderItfs
        .values()) {
      inputFiles.addAll(collectionProvider.getFiles());
    }

    linkedFile = getOutputFile();
    createParentDir(linkedFile);
    link(linkedFile, inputFiles);
  }

  // -------------------------------------------------------------------------
  // Implementation of the FileProvider interface
  // -------------------------------------------------------------------------

  public File getFile() {
    return linkedFile;
  }
}
