/***
 * Cecilia ADL Compiler
 * Copyright (C) 2006-2007 STMicroelectronics
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@objectweb.org
 *
 * Author:Ali Erdem Ozcan
 */

package org.objectweb.fractal.cecilia.adl.interfaces;

import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.objectweb.fractal.adl.ADLException;
import org.objectweb.fractal.adl.AbstractLoader;
import org.objectweb.fractal.adl.Definition;
import org.objectweb.fractal.adl.Loader;
import org.objectweb.fractal.adl.Node;
import org.objectweb.fractal.adl.components.Component;
import org.objectweb.fractal.adl.components.ComponentContainer;
import org.objectweb.fractal.adl.interfaces.IDLException;
import org.objectweb.fractal.adl.interfaces.IDLLoader;
import org.objectweb.fractal.adl.interfaces.Interface;
import org.objectweb.fractal.adl.interfaces.InterfaceContainer;
import org.objectweb.fractal.adl.types.TypeInterface;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.cecilia.adl.idl.ast.IDLDefinitionContainer;
import org.objectweb.fractal.cecilia.adl.idl.ast.InterfaceDefinition;

/**
 * A {@link Loader} to check {@link Interface} nodes in definitions. This loader
 * checks:
 * <ul>
 * <li> that interface names are locally unique</li>
 * <li>loads the AST of the interface definition</li>
 * <li> merges the AST definition of the interface with the one of the ADL.</li>
 * </ul>
 */

public class InterfaceLoader extends AbstractLoader {

  /**
   * The name of the {@link IDLLoader} client interface used by this component.
   */
  public static final String INTERFACE_LOADER_ITF = "interface-loader";

  /** The {@link IDLLoader} client interface used by this component. */
  protected IDLLoader        idlLoaderItf;

  // --------------------------------------------------------------------------
  // Implementation of the Loader interface
  // --------------------------------------------------------------------------

  public Definition load(final String name, final Map<Object, Object> context)
      throws ADLException {
    final Definition d = clientLoader.load(name, context);
    checkNode(d, context);
    return d;
  }

  // --------------------------------------------------------------------------
  // Checking methods
  // --------------------------------------------------------------------------

  private void checkNode(final Object node, final Map<Object, Object> context)
      throws ADLException {
    if (node instanceof InterfaceContainer) {
      checkInterfaceContainer((InterfaceContainer) node, context);
    }
    if (node instanceof ComponentContainer) {
      for (final Component comp : ((ComponentContainer) node).getComponents()) {
        checkNode(comp, context);
      }
    }
  }

  private void checkInterfaceContainer(final InterfaceContainer container,
      final Map<Object, Object> context) throws ADLException {
    final Set<String> names = new HashSet<String>();
    // DebugHelper.printNode("checkInterfaceContainer", (Node) container);
    for (final Interface itf : container.getInterfaces()) {
      if (itf.getName() == null) {
        throw new ADLException("Interface name missing", (Node) itf);
      }
      if (names.contains(itf.getName())) {
        throw new ADLException("Duplicated interface name '" + itf.getName()
            + "'", (Node) itf);
      } else {
        names.add(itf.getName());
      }
      if (!(itf instanceof IDLDefinitionContainer)) {
        throw new ADLException("Invalid interface node. Check the DTD.",
            (Node) itf);
      }
      // Load the interface definition node
      Node interfaceDefinition = null;
      final String signature = ((TypeInterface) itf).getSignature();
      try {
        interfaceDefinition = (Node) idlLoaderItf.loadInterface(signature,
            context);
      } catch (final IDLException e) {
        throw new ADLException("Invalid interface signature '" + signature
            + "'", (Node) container, e);
      }
      // Test if the node is an instance of InterfaceDefinition
      if (!(interfaceDefinition instanceof InterfaceDefinition)) {
        throw new ADLException("Invalid signature '" + signature
            + "' : component interface must be IDL interface", (Node) container);
      }
      // Add the interface definition node into the interface node
      ((IDLDefinitionContainer) itf)
          .setIDLDefinition((InterfaceDefinition) interfaceDefinition);
    }
  }

  @Override
  public void bindFc(final String s, final Object o)
      throws NoSuchInterfaceException, IllegalBindingException {

    if (s == null) {
      throw new IllegalArgumentException("Interface name can't be null");
    }

    if (s.equals(LOADER_BINDING)) {
      this.clientLoader = (Loader) o;
    } else if (s.equals(INTERFACE_LOADER_ITF)) {
      this.idlLoaderItf = (IDLLoader) o;
    } else {
      throw new NoSuchInterfaceException("There is no interface named '" + s
          + "'");
    }
  }

  @Override
  public String[] listFc() {
    return new String[]{LOADER_BINDING, INTERFACE_LOADER_ITF};
  }

  @Override
  public Object lookupFc(final String s) throws NoSuchInterfaceException {

    if (s == null) {
      throw new IllegalArgumentException("Interface name can't be null");
    }

    if (s.equals(LOADER_BINDING)) {
      return this.clientLoader;
    } else if (s.equals(INTERFACE_LOADER_ITF)) {
      return this.idlLoaderItf;
    } else {
      throw new NoSuchInterfaceException("There is no interface named '" + s
          + "'");
    }
  }

  @Override
  public void unbindFc(final String s) throws IllegalBindingException,
      NoSuchInterfaceException {

    if (s == null) {
      throw new IllegalArgumentException("Interface name can't be null");
    }

    if (s.equals(LOADER_BINDING)) {
      this.clientLoader = null;
    } else if (s.equals(INTERFACE_LOADER_ITF)) {
      this.idlLoaderItf = null;
    } else {
      throw new NoSuchInterfaceException("There is no interface named '" + s
          + "'");
    }
  }

}
